#include <exec/types.h>
#include <intuition/gadgetclass.h>
#include <intuition/intuition.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/keymap.h>

#include <gadgets/db_button.h>
#include <proto/db_button.h>

#include <stdio.h>

struct Window *w = NULL;
struct Gadget *but_a = NULL, *but_b = NULL ,*but_c = NULL;

struct IntuitionBase *IntuitionBase = NULL;
struct Library *KeymapBase = NULL;
struct Library *DB_ButtonBase = NULL;


#define VANILLA_GARBAGE    0xffff
#define VANILLA_RAW_KEY    0

/*
   A small function that converts rawkeys to vanillakeys.
   If the return is VANILLA_RAW_KEY (0) then it couldn't
   be maped into only one char. Then you have to take care
   of the rawkey yourself. For repeat messages the function
   returns VANILLA_GARBAGE. No need for those repeat mess!
*/
UWORD RawToVanilla( struct IntuiMessage *im )
{
   struct InputEvent ievent;
   UWORD code = 0;
   int res;

   if( im->Qualifier & IEQUALIFIER_REPEAT ) return( VANILLA_GARBAGE );

   ievent.ie_NextEvent        = NULL;
   ievent.ie_Class            = IECLASS_RAWKEY;
   ievent.ie_SubClass         = 0;
   ievent.ie_Code             = im->Code & ~IECODE_UP_PREFIX;
   ievent.ie_Qualifier        = im->Qualifier;
   ievent.ie_EventAddress     = im->IAddress;

   res = MapRawKey(&ievent, ((UBYTE*)&code)+1 ,1,NULL);
   
   if( res == -1 ) code = VANILLA_RAW_KEY;

   return( code );
}



int OpenAllLibs( void )
{
   IntuitionBase  = (struct IntuitionBase *)OpenLibrary("intuition.library", 36 );
   KeymapBase     = OpenLibrary("keymap.library", 36 );
   DB_ButtonBase  = OpenLibrary("Gadgets/db_button.gadget", 0 );

   return( IntuitionBase && KeymapBase && DB_ButtonBase );
}

void CleanUp( void )
{
   if( but_c )          DisposeObject( but_c );
   if( but_b )          DisposeObject( but_b );
   if( but_a )          DisposeObject( but_a );

   if( w )              CloseWindow( w );

   if( DB_ButtonBase )  CloseLibrary( DB_ButtonBase );
   if( KeymapBase )     CloseLibrary( KeymapBase );
   if( IntuitionBase )  CloseLibrary( (struct Library *) IntuitionBase );
}

void main( void )
{
   BOOL keepgoing = TRUE;
   struct IntuiMessage *msg;
   UWORD code;
   int button_height;
   struct Screen *scr;

   if( ! OpenAllLibs() ) goto abort;

   scr = LockPubScreen(NULL);

   if( ! scr ) goto abort;

   button_height = scr->Font->ta_YSize + 6;

   w = OpenWindowTags( NULL, WA_Flags,       WFLG_DEPTHGADGET | WFLG_DRAGBAR |
                                             WFLG_CLOSEGADGET,
                             WA_IDCMP,       IDCMP_CLOSEWINDOW | IDCMP_RAWKEY,
                             WA_Top,         15,
                             WA_Left,        150,
                             WA_InnerWidth,  160,
                             WA_InnerHeight, button_height*3 + 30,
                             WA_Activate,    TRUE,
                             WA_PubScreen,   scr,
                             TAG_END);

   UnlockPubScreen( NULL, scr );

   if( !w ) goto abort;

   but_a = NewObject( dbBT_GetClass(), NULL,
                      GA_ID,              1,
                      GA_Top,             w->BorderTop + 7,
                      GA_Left,            w->BorderLeft + 5,
                      GA_Width,           150,
                      GA_Height,          button_height,
                      DBBT_Text,          "Button _a",
                      DBBT_Underscore,    '_',
                      TAG_DONE);

   if( !but_a ) goto abort;

   but_b = NewObject( dbBT_GetClass(), NULL,
                      GA_ID,              2,
                      GA_Top,             but_a->TopEdge + but_a->Height + 7,
                      GA_Left,            w->BorderLeft + 5,
                      GA_Width,           150,
                      GA_Height,          button_height,
                      GA_Previous,        but_a,
                      DBBT_Text,          "Button _b",
                      DBBT_Underscore,    '_',
                      TAG_DONE);

   if( !but_b ) goto abort;

   but_c = NewObject( dbBT_GetClass(), NULL,
                      GA_ID,              3,
                      GA_Top,             but_b->TopEdge + but_b->Height + 7,
                      GA_Left,            w->BorderLeft + 5,
                      GA_Width,           150,
                      GA_Height,          button_height,
                      GA_Previous,        but_b,
                      DBBT_Text,          "Button _c",
                      DBBT_Underscore,    '_',
                      DBBT_SoftStyle,     FSF_BOLD,
                      TAG_DONE);

   if( !but_c ) goto abort;

   AddGList( w, but_a, -1, -1, NULL );
   RefreshGList( but_a, w, NULL, -1 );

   while( keepgoing )
   {
      WaitPort( w->UserPort );
      while( msg = (struct IntuiMessage *)GetMsg(w->UserPort) )
      {
         switch( msg->Class )
         {
            case IDCMP_CLOSEWINDOW :
               keepgoing = FALSE;
               break;
            case IDCMP_RAWKEY :
               code = RawToVanilla( msg );
               switch( code )
               {
                  case 'a' :
                  case 'A' :
                     dbBT_Press( but_a, w, NULL, msg );
                     break;
                  case 'b' :
                  case 'B' :
                     dbBT_Press( but_b, w, NULL, msg );
                     break;
                  case 'c' :
                  case 'C' :
                  case 0xD : // ENTER
                     dbBT_Press( but_c, w, NULL, msg );
                     break;
                  case 0x1B : //ESC
                     keepgoing = FALSE;
                     break;
                  default :
                     break;
               }
               break;
            default : break;
         }
         
         ReplyMsg( (struct Message *) msg );
      }
   }

   RemoveGList( w, but_a, -1 );

abort:
   CleanUp();
}
